/*
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * you may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.ballerinalang.model.util.serializer.providers.bvalue;

import org.ballerinalang.model.util.serializer.BPacket;
import org.ballerinalang.model.util.serializer.BValueDeserializer;
import org.ballerinalang.model.util.serializer.BValueSerializer;
import org.ballerinalang.model.util.serializer.SerializationBValueProvider;
import org.ballerinalang.model.values.BBoolean;
import org.ballerinalang.model.values.BValue;

/**
 * Provide mapping between {@link Class} and {@link BValue} representation of it.
 *
 * @since 0.982.0
 */
public class BBooleanBValueProvider implements SerializationBValueProvider<BBoolean> {
    @Override
    public String typeName() {
        return getType().getSimpleName();
    }

    @Override
    public Class<?> getType() {
        return BBoolean.class;
    }

    @Override
    public BPacket toBValue(BBoolean bBoolean, BValueSerializer serializer) {
        return BPacket.from(typeName(), bBoolean);
    }

    @Override
    public BBoolean toObject(BPacket packet, BValueDeserializer bValueDeserializer) {
        return (BBoolean) packet.getValue();
    }
}
